/**
 * 垂直折线的绘图区 LinePlotGlyph.start
 *
 *      @example
 *      var linePlot = new FR.Chart.LinePlot();
 *
 * @class FR.Chart.LinePlotGlyph
 * @extends FR.Chart.CategoryPlotGlyph
 *
 * @param {JSON} js 配置属性
 *
 * @param {Boolean} [js.isCurve=false] 是否曲线连接
 */
FR.Chart.VerticalLinePlotGlyph = FR.extend(FR.Chart.LinePlotGlyph, {
    initConstants: function (js) {
        js = js || {};

        FR.Chart.LinePlotGlyph.superclass.initConstants.apply(this, [js]);

    },

    /**
     * 布局大数据系列,先从年往下一次，年，月，日，时，分，秒
     */
    dealLine4StockChartLargeData: function (dataSeries, categoryPlotGlyph, xAxisGlyph, yAxisGlyph,adjustType) {

        for (var categoryIndex = 0; categoryIndex < dataSeries.getDataPointCount(); categoryIndex++) {
            dataSeries.points[categoryIndex].revert2Original();
        }

        var isSeriesCurve = this.isSeriesCurve(dataSeries);

        var currentCombineDate = null;
        var combineDataPoint = null;
        var currentCombineX = -1;

        var stockDataPoints = [];


        //调整合并
        for (var categoryIndex = 0; categoryIndex < dataSeries.getDataPointCount(); categoryIndex++) {
            var dataPoint = dataSeries.points[categoryIndex];
            if (dataPoint.valueIsNull) {
                dataPoint.glyph = null;
                continue;
            }

            var dataPointX = xAxisGlyph.getPoint2D(xAxisGlyph.getTickIndex4Value(categoryIndex)).x;
            if (this.isDataPointXNotInPlotBounds(dataPointX)) {//kunsnat: 超出 左侧和右侧范围 不进入统计. 针对IE 但是上下两侧无法处理, 需要确认.
                dataPoint.glyph = null;
                continue;
            }

            var cateDate = xAxisGlyph.getDate4Value(categoryIndex);
            if (stockDataPoints.length === 0) {
                currentCombineDate = cateDate;
                stockDataPoints[0] = dataPoint;
                combineDataPoint = dataPoint;
                dataPoint.categoryName = FR.Chart.ChartUtils.calculateAdjustDateRange(cateDate,adjustType);
                combineDataPoint.stockDataPointCount = 1;
                currentCombineX = parseInt(dataPointX);
                continue;
            } else if (FR.Chart.ChartUtils.isBelongToSameUnit(cateDate, currentCombineDate, adjustType)) {//同一范围数值累加
                combineDataPoint.addDataPoint4StockCategory(dataPoint);
                combineDataPoint.stockDataPointCount++;
                dataPoint.valueIsNull = true;
                continue;
            }


            var stockIndex = this.getStockDateRangeIndex(stockDataPoints, cateDate, xAxisGlyph, adjustType);
            //是否属于之前的一个统计区间
            if (stockIndex !== -1) {
                stockDataPoints[stockIndex].addDataPoint4StockCategory(dataPoint);
                dataPoint.valueIsNull = true;
                combineDataPoint.stockDataPointCount++;
            } else { //属于新的统计区间
                currentCombineX = parseInt(dataPointX);
                dataPoint.categoryName = FR.Chart.ChartUtils.calculateAdjustDateRange(cateDate,adjustType);
                stockDataPoints[stockDataPoints.length] = dataPoint;
                currentCombineDate = cateDate;
                combineDataPoint = dataPoint;
                combineDataPoint.stockDataPointCount = 1;
                currentCombineX = parseInt(dataPointX);
            }
        }

        //折线图面积图stock chart调整区间内是以平均值来展示的
        var stockCount = stockDataPoints.length;
        for (var stockIndex = 0; stockIndex < stockCount; stockIndex++) {
            var stockPoint = stockDataPoints[stockIndex];
            stockPoint.value = stockPoint.value / stockPoint.stockDataPointCount;
            stockPoint.percentValue = stockPoint.percentValue / stockPoint.stockDataPointCount;
        }


        //刷新值轴
        yAxisGlyph.oldMin = yAxisGlyph.minValue;
        yAxisGlyph.oldMax = yAxisGlyph.maxValue;
        yAxisGlyph.oldTick = yAxisGlyph.tickLength;

        yAxisGlyph.initMinMaxValue(this.getMinValueFromData(yAxisGlyph.isLog, FR.Chart.CustomAttr.Axis_LEFT),
            this.getMaxValueFromData(FR.Chart.CustomAttr.Axis_LEFT));
        yAxisGlyph.calculateAxisGlyph(this.bounds);
        xAxisGlyph.stockChartMainType = adjustType;


        //布局图形
        var xVal = [];
        var yVal = [];
        var lineDataCount = 0;
        var isStartPoint = true;
        var linePaths = new FR.Chart.GeneralPath();
        var curvePaths = new FR.Chart.GeneralPath();
        if (xAxisGlyph.axisReversed) {
            linePaths.axisReversed = true;
            curvePaths.axisReversed = true;
        }


        var plotBounds = this.getBounds();
        for (var stockIndex = 0; stockIndex < stockCount; stockIndex++) {
            var stockPoint = stockDataPoints[stockIndex];

            var point_x = this.getStockChartDatePointX(xAxisGlyph,stockPoint,adjustType,this.getBounds());
            var point_y = yAxisGlyph.getPoint2D(stockPoint.value).y;

            xVal[lineDataCount] = point_x;
            yVal[lineDataCount] = point_y;
            var last_point_y = isStartPoint ? 0 : yVal[lineDataCount - 1];
            lineDataCount++;

            this.deal4LinePath(isSeriesCurve,isStartPoint,linePaths,curvePaths,point_x,point_y,last_point_y,stockPoint);
            if(isStartPoint){
                isStartPoint = false;
            }
        }

        if (isSeriesCurve) {
            FR.Chart.FoldLine.curveTo(linePaths, curvePaths);
        }

        var foldLine = new FR.Chart.FoldLine(linePaths);
        dataSeries.glyph = foldLine;

        foldLine.lineStyleInfo.dealCondition(this.conditionCollection, dataSeries, this.createColors4Series());

        if (foldLine.lineStyleInfo.attrLineColor.seriesColor) {
            var tipBackground = new FR.Chart.ColorBackground();
            tipBackground.color = foldLine.lineStyleInfo.attrLineColor.seriesColor;
            foldLine.toolTipBackground = tipBackground;
        }

        this.trendLineFitting(xVal, yVal, dataSeries, yAxisGlyph);
    },


    /**
     * 处理折线的绘图区系列.
     * @param dataSeries 系列
     * @param categoryPlotGlyph 分类绘图区.
     * @param xAxisGlyph x轴
     * @param yAxisGlyph y轴
     */
    dealLine4PlotGlyph: function (dataSeries, categoryPlotGlyph, xAxisGlyph, yAxisGlyph) {
        var xVal = [];
        var yVal = [];
        var lineDataCount = 0;

        xAxisGlyph.stockChartMinDateInt = null;
        xAxisGlyph.stockChartMainType = null;

        var linePaths = new FR.Chart.GeneralPath();
        var curvePaths = new FR.Chart.GeneralPath();
        if (xAxisGlyph.axisReversed) {
            linePaths.axisReversed = true;
            curvePaths.axisReversed = true;
        }

        if(this.isStockChart){
            for (var categoryIndex = 0; categoryIndex < dataSeries.getDataPointCount(); categoryIndex++) {
                dataSeries.points[categoryIndex].revert2Original();
            }
        }

        var isStartPoint = true;
        var isSeriesCurve = this.isSeriesCurve(dataSeries);
        var isSeriesNullValueBreak = this.isSeriesNullValueBreak(dataSeries);

        var lastPointX = -1;
        var lastDate = null;

        for (var categoryIndex = 0; categoryIndex < dataSeries.getDataPointCount(); categoryIndex++) {
            var dataPoint = dataSeries.points[categoryIndex];

            if (dataPoint.valueIsNull) {
                if (isSeriesNullValueBreak) {
                    isStartPoint = true;  //空值断开
                }
                continue;
            }
            var isZoom = xAxisGlyph.zoomBackward !== 0 || xAxisGlyph.zoomForward !== 0;
            var value = categoryPlotGlyph.getDataPointPercentValue(dataSeries.seriesIndex, categoryIndex);
            var point_x = xAxisGlyph.getPoint2D(xAxisGlyph.getTickIndex4Value(categoryIndex)).x;
            if (this.isDataPointXNotInPlotBounds(point_x)) {//kunsnat: 超出 左侧和右侧范围 不进入统计. 针对IE 但是上下两侧无法处理, 需要确认.
                dataPoint.glyph = null;
                if(!isZoom || (isZoom && dataPoint.isOutBounds)){
                    dataPoint.isOutBounds  = true;
                    continue;
                }
            }

            if(this.isStockChart && lastPointX === parseInt(point_x)){
                //纪录一下是在年月日还是时分秒重叠,返回下面要用的时间单位
                var cateDate =xAxisGlyph.getDate4Value(categoryIndex);
                return this.getStockChartAdjustDateType(cateDate,lastDate);
            }else{
                lastPointX = parseInt(point_x);
            }

            if(this.isStockChart){
                lastDate = xAxisGlyph.getDate4Value(categoryIndex);
            }

            var point_y = yAxisGlyph.getPoint2D(value).y;

            xVal[lineDataCount] = point_x;
            yVal[lineDataCount] = point_y;
            var last_point_y = isStartPoint ? 0 : yVal[lineDataCount - 1];
            lineDataCount++;

            this.deal4LinePath(isSeriesCurve,isStartPoint,linePaths,curvePaths,point_x,point_y,last_point_y,dataPoint);
            if(isStartPoint){
                isStartPoint = false;
            }
        }

        if (isSeriesCurve) {
            FR.Chart.FoldLine.curveTo(linePaths, curvePaths);
        }

        var foldLine = new FR.Chart.FoldLine(linePaths);
        dataSeries.glyph = foldLine;

        foldLine.lineStyleInfo.dealCondition(this.conditionCollection, dataSeries, this.createColors4Series());

        if (foldLine.lineStyleInfo.attrLineColor.seriesColor) {
            var tipBackground = new FR.Chart.ColorBackground();
            tipBackground.color = foldLine.lineStyleInfo.attrLineColor.seriesColor;
            foldLine.toolTipBackground = tipBackground;
        }

        this.trendLineFitting(xVal, yVal, dataSeries, yAxisGlyph);
        return -1;
    },


    deal4LinePath:function(isSeriesCurve,isStartPoint,linePaths,curvePaths,pointX,pointY,lastPointY,dataPoint){
        if (isStartPoint) {
            linePaths.moveTo(pointX, pointY);
        } else {
            linePaths.lineTo(pointX,lastPointY);
            linePaths.lineTo(pointX, pointY);
        }

        if (!this.isDataPointXNotInPlotBounds(pointX)) {
            var markerGlyph = new FR.Chart.MarkerGlyph();
            markerGlyph.dealCondition4Line(this.conditionCollection, dataPoint, this.createColors4Series());
            markerGlyph.setShape(new FR.Chart.Bounds(pointX - 5, pointY - 5, 10, 10));
            markerGlyph.background = this.getBackground() !== null ? this.getBackground():this.wholeChartBackground;
            dataPoint.glyph = markerGlyph;
            markerGlyph.toolTipBackground = markerGlyph.background;

            this.dealDataPointLabel(dataPoint);
        }
    }

});